{% extends "base.html" %}
{% block website_header %}{{ post.title }} | {{website_name}}{% endblock %}

{% block content %}
<article class="post">
    <h2>{{ post.title }}</h2>
    {{ content }}
</article>
{% endblock %}
