@charset "utf-8";

body    { font: 17px/25px Verdana, sans-serif;
          font-weight: normal;
          margin: auto;
          max-width: 880px;
          padding: 0 auto;  
          color: black;}
a       { color: black; }
a:hover { color: black; }

/* navigation */
header { padding: 20px 20px; margin-bottom: 40px; display: flex; justify-content: space-between;}
header nav ul { margin:0; padding:0;} /* We have to reset hard these properties as they have non-zero defaults*/
header nav ul li { display: inline; padding: 0 5px 0 5px }

/* headlines */
h1, h2, h3, h4, h5, h6      { font-family: Georgia, serif;
                              font-weight: bold; 
                              color: black;}

/* main */
main { margin: 0; padding: 0 75px 20px 75px}

/* posts */
article.post {margin: auto;}
article.post p { text-align: justify;}
section.post_lists { margin: auto; }
section div.post_header_container { display: flex; justify-content: flex-start; align-items: baseline;}
section div.post_header_date {padding-right: 10px; flex-basis: 30% ; text-align: right; font-style: italic;}
section div.post_header_summary h3 { font-weight: normal; }
section.tags {word-spacing: 20px;}

/* pagination */
section.pagination { text-align: center}

/* code */
code {font-family: monospace; font-size: 12px;}

/* footer */
footer { margin: 0; 
         padding: 0 15px;
         text-align: right;}
