<!DOCTYPE html>
<html lang="en">

<head>
    <meta charset="UTF-8"/>
    {% block head %}
    <title>{% block website_header %}{% endblock %}</title>
    {% if website_description %}
    <meta name="description" content="{{ website_description }}"/>
    {% endif %}
    {% if website_author %}
    <meta name="author" content="{{ website_author }}"/>
    {% endif %}
    {% if website_keywords %}
    <meta name="keywords" content="{{ website_keywords }}"/>
    {% endif %}
    <meta name="viewport" content="width=device-width, initial-scale=1.0"/>
    {% if website_root %}
    <base href="{{ website_root }}">
    {% else %}
    <base href="/">
    {% endif %}
    <link rel="stylesheet" href="style.css"/>

    {% endblock %}
</head>

<body>

<header>
    <div id="title">
        <a href="">{{ website_name }}</a>
    </div>
    <nav>
        <ul>
            <li><a href="">Home</a></li>
            <li><a href="archive.html">Archive</a></li>
            <li><a href="tags.html">Tags</a></li>
        </ul>
    </nav>
</header>

<main>
    {% block content %}
    {% endblock %}
</main>

<footer>
    {% if website_author %}
    <p>&copy; Copyright {{ current_year }} by {{ website_author }} </p>
    {% else %}
    <p>&copy; Copyright {{ current_year }}</p>
    {% endif %}
</footer>

</body>

</html>
